@@ -2,27 +2,14 @@
 base class for python applications with a graphical user interface
 ==================================================================
 
-The abstract base class :class:`MainAppBase` provided by this ae namespace
-portion can be used for to integrate any of the available Python GUI
-frameworks into the ae namespace.
+The abstract base class :class:`MainAppBase` provided by this
+ae namespace portion allows the integration of any Python GUI
+framework into the ae namespace.
 
-The plan is to integrate the following GUI frameworks until the begin of 2021:
-
-* :mod:`Kivy <ae.kivy_app>` - see also :ref:`kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>`
-* :mod:`Enaml <ae.enaml_app>` - see also :ref:`enaml lisz demo app <https://gitlab.com/ae-group/enaml_lisz>`
-* :mod:`Beeware <ae.beeware_app>`
-* :mod:`QPython <ae.qpython_app>`
-* :mod:`pyglet <ae.pyglet_app>`
-* :mod:`pygobject <ae.pygobject_app>`
-* :mod:`Dabo <ae.dabo_app>`
-* :mod:`AppJar <ae.appjar_app>`
-
-Currently available is the :mod:`Kivy <ae.kivy_app>` integration of
-the :ref:`Kivy Framework <kivy.org>`.
-
-.. note:
-    In implementing the outstanding framework integrations this module will be
-    extended and changed frequently.
+On overview about the available GUI-framework-specific
+ae namespace portion implementations can be found
+in the documentation of the ae namespace portion
+:mod:`ae.lisz_app_data`.
 
 
 extended console application environment
@@ -174,65 +161,70 @@ actions defined in :data:`ACTIONS_REDUCING_FLOW_PATH`.
 application events
 ------------------
 
-The helper method :meth:`~MainAppBase.call_method` can be used to support
-optionally implemented event callback routines.
+The following application events are fired exactly
+one time in the lifetime of an app instance in
+the following order:
 
-An implementation for a GUI framework should provide at least the
-following application events (which have to be fired exactly
-one time in the runtime of the app):
+* `on_app_init`: fired **after** :class:`ConsoleApp`-parts
+  of the app instance got initialized (loaded config files
+  and app states)
+  and **before** the resources got loaded and the GUI
+  framework app class instance gets initialized.
+* `on_app_start`: fired **after** the call of the
+  method :meth:`~MainAppBase.run_app`
+  and **before** the command line options got parsed.
+* `on_app_run`: fired **after** the parsing of the
+  command line arguments and options and
+  **before** the framework event
+  loop of the used GUI framework gets started.
+* `on_app_stop`: fired **after** the call of the
+  method :meth:`~MainAppBase.stop_app`
+  and **before** the framework win will
+  be closed.
+* `on_app_exit`: fired **after* framework win got closed and
+  just **before** the event loop of the GUI framework
+  will be stopped and the app shutdown.
+* `on_app_quit`: fired **after* the event loop of the GUI framework
+  got stopped and before the :meth:`AppBase.shutdown` method
+  will be called.
 
-* `on_app_init`: fired after the resources got loaded (normally done by the
-  method :meth:`init_app` which gets implemented by the inheriting app class layer)
-  and before the GUI framework app class instance gets initialized.
-* `on_app_start`: fired after the framework app and window classes got initialized
-  and the first window will be displayed (mostly by calling the `show` method
-  of the window, but this is depending on the finally used GUI framework)
-  and before the event loop of the used GUI framework got started.
-* `on_app_run`: fired directly after the window got shown and framework event
-  loop of the used GUI framework got started.
-* `on_app_stop`: fired after framework win got closed and just before the
-  event loop of the GUI framework will be stopped.
-
-.. hint::
-    Some lazy/late initialization are possibly not done when the `on_app_init` or
-    `on_app_start` events gets fired.
-    E.g. if you want to display a debug log/console output using the
-    :meth:`~ConsoleApp.debug_print` or :meth:`~ConsoleApp.dpo` method and you did not
-    call explicitly :meth:`ConsoleApp._parse_args` before, then the value of the
-    `debug_level` config variable not got already loaded into the instance attribute
-    :attr:`~AppBase.debug_level` and you will see no debug output.
+.. note::
+    The `on_app_stop` and `on_app_exit` events will only be
+    fired if the app is explicitly calling the
+    :meth:`~MainAppBase.stop_app` method.
 
 
 application state change events
 _______________________________
 
-The method :meth:`~MainAppBase.call_method` is also used internally by this
-module for to fire notification events when one of the app state variables
+Notification events get fired when one of the app state variables
 gets changed - like e.g. the :ref:`application flow` or if
 the user changes the font size.
 
-The method name of the notification event consists of the prefix ``on_`` followed
-by the variable name of the app state. So e.g. on a change of the `font_size` the
-notification event `on_font_size` will be fired/called (if exists as a method
+The method name of the notification event consists of
+the prefix ``on_`` followed by the variable name of the app state.
+So e.g. on a change of the `font_size` app state the notification
+event `on_font_size` will be fired/called (if exists as a method
 of the main app instance).
 
 
 application flow change events
 ______________________________
 
-When the method :meth:`~MainAppBase.change_flow` get called for to change
-the current flow, then first a flow-specific event gets fired for to
-confirm the flow id change. The name of this event depends
-on the new flow id and gets determined by the function
-:func:`flow_event_name`. If the event handler exists and
-does return `True` for to confirm the flow change then flow id and path
+The name of the event method that gets fired when the app want
+to change the flow (via the method :meth:`~MainAppBase.change_flow`)
+depends on the new flow id and gets determined by the function
+:func:`flow_event_name`.
+
+If the event handler exists and does return `True` for to
+confirm the flow change then flow id and path
 will be changed accordingly.
 
 If the flow-specific event call results in a boolean `False` or `None`
-then the generic event handler :meth:`~MainAppBase.on_flow_change`
+then a more generic event :meth:`~MainAppBase.on_flow_change`
 will be called.
 
-If also the generic event handler returns `False` then the action
+If also the generic event returns `False` then the action
 of the new flow id will be searched within
 :data:`ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION` and if not found
 then the flow change will be rejected and :meth:`~MainAppBase.change_flow`
@@ -366,7 +358,6 @@ either the inheriting framework portion or directly by user main app
 class.
 """
 from abc import ABC, abstractmethod
-from configparser import NoSectionError
 import re
 import traceback
 from typing import Any, Callable, Dict, Tuple, List, Optional, Type, Sequence
@@ -378,7 +369,7 @@ from ae.console import ConsoleApp                   # type: ignore
 from ae.files import FilesRegister, RegisteredFile  # type: ignore
 
 
-__version__ = '0.0.19'
+__version__ = '0.0.20'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -542,18 +533,21 @@ class MainAppBase(ConsoleApp, ABC):
         super().__init__(**console_app_kwargs)
 
         self.load_app_states()
+
+        self.call_method('on_app_init')
+
         self.load_images()
         self.load_sounds()
 
         self._start_event_loop, self._stop_event_loop = self.init_app()
 
     @abstractmethod
-    def init_app(self, **kwargs) -> Tuple[Optional[Callable], Optional[Callable]]:
+    def init_app(self, framework_app_class: Any = None) -> Tuple[Optional[Callable], Optional[Callable]]:
         """ initialize framework app instance and root window/layout, return GUI event loop start/stop methods.
 
-        :param kwargs:  optional arguments used by app project to inject e.g. classes that got extended
-                        by the app project for to create the framework_app/framework_win instances.
-        :return:        tuple of two callable, the 1st for to start and the 2nd for to stop/exit the GUI event loop.
+        :param framework_app_class:     class to create app instance (optionally extended by app project).
+        :return:                        tuple of two callable, the 1st for to start and the 2nd for to stop/exit
+                                        the GUI event loop.
         """
 
     # app state helper methods
@@ -563,11 +557,7 @@ class MainAppBase(ConsoleApp, ABC):
 
         :return:                tuple of all app state item keys (config variable names).
         """
-        try:  # quicker than asking before with: if cfg_parser.has_section(APP_STATE_SECTION_NAME):
-            return tuple(self._cfg_parser.options(APP_STATE_SECTION_NAME))
-        except NoSectionError:
-            self.dpo(f"MainAppBase.app_state_keys: ignoring missing config file section {APP_STATE_SECTION_NAME}")
-            return tuple()
+        return self.cfg_section_variable_names(APP_STATE_SECTION_NAME)
 
     def change_app_state(self, state_name: str, new_value: Any, send_event: bool = True):
         """ change single app state item to value in self.attribute and app_state dict item.
@@ -870,15 +860,6 @@ class MainAppBase(ConsoleApp, ABC):
         """ load audio sounds from app folder snd. """
         self.sound_files = FilesRegister('snd')
 
-    def on_app_run(self):
-        """ event fired after framework event loop got started. """
-        self.dpo("MainAppBase.on_app_run")
-        self.change_flow(self.flow_id)
-
-    def on_app_stop(self):
-        """ event fired after framework win got closed and just before the event loop will be stopped. """
-        self.dpo("MainAppBase.on_app_stop")
-
     def play_beep(self):
         """ make a short beep sound, should be overwritten by GUI framework. """
         self.po(chr(7), "MainAppBase.BEEP")
@@ -901,13 +882,17 @@ class MainAppBase(ConsoleApp, ABC):
         """ startup main and framework applications. """
         self.dpo("MainAppBase.run_app")
 
-        if not self._parsed_args:
-            self._parse_args()
+        self.call_method('on_app_start')
+
+        super().run_app()                               # parse command line arguments into config options
+
+        self.call_method('on_app_run')
 
         if self._start_event_loop:                      # not needed for SubApp or additional Window instances
             try:
                 self._start_event_loop()
             finally:
+                self.call_method('on_app_quit')
                 self.shutdown(self._exit_code or None)  # don't call sys.exit() for zero exit code
 
     def show_popup(self, popup_class: Type, **popup_kwargs) -> Any:
@@ -933,9 +918,13 @@ class MainAppBase(ConsoleApp, ABC):
         self.dpo(f"MainAppBase.stop_app {exit_code}")
         self._exit_code = exit_code
 
+        self.call_method('on_app_stop')
+
         if self.framework_win:
             self.framework_win.close()      # close window to save app state data
 
+        self.call_method('on_app_exit')
+
         if self._stop_event_loop:
             self._stop_event_loop()         # will exit the self._start_event_loop() method called by self.run_app()
 
